#version 330

struct MaterialParam {
  vec4 ambient;
  vec4 diffuse;
  vec4 specular;
  float shininess;
};

uniform vec3          u_lightPos;
uniform MaterialParam u_lightMaterial;
uniform MaterialParam u_material;

uniform mat4 u_modelMatrix;        // モデル・マトリックス
uniform mat4 u_viewMatrix;         // ビュー・マトリックス
uniform mat4 u_projectionMatrix;   // 射影・マトリックス

in vec3 a_position;
in vec3 a_normal;

flat out vec4 v_color;


void main(void)
{
  mat4 modelViewMatrix = u_viewMatrix * u_modelMatrix;
  vec3 lightPos = vec3(u_viewMatrix * vec4(u_lightPos.xyz,1.0));
  mat3 n_mat = mat3( transpose( inverse(modelViewMatrix) ) ); // normal Matrix

  vec3 P = vec3(modelViewMatrix * vec4(a_position,1.0));
  vec3 N = normalize(n_mat * a_normal);
  vec3 L = normalize(lightPos.xyz - P);  // light vector
  float nDotL = dot(N,L);
  float diffuseLight = max(nDotL,0.0);

  vec4 ambient = u_lightMaterial.ambient * u_material.ambient;
  vec4 diffuse = u_lightMaterial.diffuse * u_material.diffuse * diffuseLight;

  vec3 V = normalize(-P);
  //vec3 H = normalize(L+V); // ハーフベクトルによるスペキュラー
  //float nDotH = pow(max(dot(N,H),0.0), u_material.shininess);
  vec3 H = reflect(-L,N); // 反射ベクトルによるスペキュラー
  float nDotH = pow(max(dot(V,H),0.0), u_material.shininess);

  if (nDotH<0.0) nDotH = 0.0;
  vec4 specular = u_lightMaterial.specular * u_material.specular * nDotH;

  v_color = ambient + diffuse + specular;
  gl_Position = u_projectionMatrix*modelViewMatrix*vec4(a_position.xyz, 1.0);
}


